@@ -460,6 +460,30 @@ void pool_clear_sent_message_list(void)
 	}
 }
 
+/*
+ * Zap query context info in sent messages to indicate that the query context
+ * has been already removed.
+ */
+void
+pool_zap_query_context_in_sent_messages(POOL_QUERY_CONTEXT *query_context)
+{
+	int			i;
+	POOL_SENT_MESSAGE_LIST *msglist;
+
+	msglist = &pool_get_session_context(false)->message_list;
+
+	for (i = 0; i < msglist->size; i++)
+	{
+		elog(DEBUG5, "checking zapping sent message: %p query_context: %p",
+			 &msglist->sent_messages[i], msglist->sent_messages[i]->query_context);
+		if (msglist->sent_messages[i]->query_context == query_context)
+		{
+			msglist->sent_messages[i]->query_context = NULL;
+			elog(DEBUG5, "Zap sent message: %p", &msglist->sent_messages[i]);
+		}
+	}
+}
+
 static void dump_sent_message(char *caller, POOL_SENT_MESSAGE *m)
 {
 	ereport(DEBUG1,
@@ -569,7 +593,7 @@ void pool_add_sent_message(POOL_SENT_MESSAGE *message)
 }
 
 /*
- * Get a sent message
+ * Find a sent message by kind and name.
  */
 POOL_SENT_MESSAGE *pool_get_sent_message(char kind, const char *name, POOL_SENT_MESSAGE_STATE state)
 {
@@ -592,6 +616,29 @@ POOL_SENT_MESSAGE *pool_get_sent_message(char kind, const char *name, POOL_SENT_
 	return NULL;
 }
 
+/*
+ * Find a sent message by query context.
+ */
+POOL_SENT_MESSAGE *
+pool_get_sent_message_by_query_context(POOL_QUERY_CONTEXT * query_context)
+{
+	int			i;
+	POOL_SENT_MESSAGE_LIST *msglist;
+
+	msglist = &pool_get_session_context(false)->message_list;
+
+	if (query_context == NULL)
+		return NULL;
+
+	for (i = 0; i < msglist->size; i++)
+	{
+		if (msglist->sent_messages[i]->query_context == query_context)
+			return msglist->sent_messages[i];
+	}
+
+	return NULL;
+}
+
 /*
  * Set message state to POOL_SENT_MESSAGE_STATE to POOL_SENT_MESSAGE_CLOSED.
  */
@@ -1346,7 +1393,7 @@ void pool_pending_message_reset_previous_message(void)
 				(errmsg("pool_pending_message_reset_previous_message: session context is not initialized")));
 		return;
 	}
-	session_context->previous_message = NULL;
+	session_context->previous_message_exists = false;
 }
 
 /*
@@ -1360,11 +1407,13 @@ void pool_pending_message_set_previous_message(POOL_PENDING_MESSAGE *message)
 				(errmsg("pool_pending_message_set_previous_message: session context is not initialized")));
 		return;
 	}
-	session_context->previous_message = message;
+	session_context->previous_message_exists = true;
+	memcpy(&session_context->previous_message, message, sizeof(POOL_PENDING_MESSAGE));
 }
 
 /*
- * Get previous message.
+ * Get previous message. This actually returns the address of memory. Do not
+ * try to free using pool_pending_message_free_pending_message().
  */
 POOL_PENDING_MESSAGE *pool_pending_message_get_previous_message(void)
 {
@@ -1374,7 +1423,10 @@ POOL_PENDING_MESSAGE *pool_pending_message_get_previous_message(void)
 				(errmsg("pool_pending_message_get_previous_message: session context is not initialized")));
 		return NULL;
 	}
-	return session_context->previous_message;
+	if (session_context->previous_message_exists == false)
+		return NULL;
+
+	return &session_context->previous_message;
 }
 
 /*
@@ -1469,6 +1521,35 @@ int pool_pending_message_get_target_backend_id(POOL_PENDING_MESSAGE *msg)
 	return backend_id;
 }
 
+/*
+ * Get number of pending message list entries of which target backend is same as specified one.
+ */
+int
+pool_pending_message_get_message_num_by_backend_id(int backend_id)
+{
+	ListCell   *cell;
+	ListCell   *next;
+	int        cnt = 0;
+
+	if (!session_context)
+	{
+		ereport(ERROR,
+				(errmsg("pool_pending_message_get_message_num_by_backend_id: session context is not initialized")));
+		return 0;
+	}
+
+	for (cell = list_head(session_context->pending_messages); cell; cell = next)
+	{
+		POOL_PENDING_MESSAGE *msg = (POOL_PENDING_MESSAGE *) lfirst(cell);
+
+		if (msg->node_ids[0] == backend_id || msg->node_ids[1] == backend_id) 
+			cnt++;
+
+		next = lnext(cell);
+	}
+	return cnt;
+}
+
 /*
  * Dump whole pending message list
  */
